@@ -1,17 +1,17 @@
-# --------------------------------------------------
-# Copyright The IETF Trust 2011, All Rights Reserved
-# --------------------------------------------------
+# Copyright The IETF Trust 2018, All Rights Reserved
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals, print_function, division
 
-# Internal utitlity functions.  Not meant for public usage.
+# Internal utility functions.  Not meant for public usage.
 
 import base64
-import calendar
-import math
 import re
 import six
+import sys
 import textwrap
 
-from lxml.etree import _Comment
+from collections import OrderedDict
+from lxml.etree import _Comment, _ProcessingInstruction
 
 if six.PY2:
     from urllib import quote
@@ -19,13 +19,17 @@ else:
     from urllib.request import quote
 
 try:
-    import debug
+    from xml2rfc import debug
     assert debug
 except ImportError:
     pass
 
 import xml2rfc.log
 
+
+# ----------------------------------------------------------------------
+# Text wrapping
+
 class TextWrapper(textwrap.TextWrapper):
     """ Subclass that overrides a few things in the standard implementation """
     def __init__(self, **kwargs):
@@ -37,8 +41,9 @@ class TextWrapper(textwrap.TextWrapper):
             u'[ \t\n\r\f\v]+|'                                  # any ASCII whitespace
             u'[^\\s-]*\\w+/(?=[A-Za-z]\\w*)|'                   # forward-slash separated words
             u'[^\\s-]*\\w+[^0-9\\s]-(?=\\w+[^0-9\\s])|'         # hyphenated words
+            u'\u200b|'                                          # &zwsp; zero-width space is breakable space
             u'''(?<=[\\w\\!"'\\&\\.\\,\\?])-{2,}(?=\\w))'''     # em-dash
-            u'(?![\u2060|\u200B])')                             # UNLESS &wj; or &zwbs; or &nbsp;
+            u'(?![\u00A0|\u2060|\uE060])')                      # UNLESS &nbsp; or &wj; 
 
         self.wordsep_re_uni = re.compile(self.wordsep_re.pattern, re.U)
 
@@ -66,15 +71,6 @@ class TextWrapper(textwrap.TextWrapper):
         # Start of next sentence regex
         self.sentence_start_re = re.compile("^[\"'([]*[A-Z]")
 
-        # XmlCharRef replacements that occur AFTER line breaking logic
-        self.post_break_replacements = {
-            u'\u2060': '',    # wj
-            u'\u200B': '',    # zwsp
-        }
-
-        self.break_on_hyphens = True
-
-
     def _fix_sentence_endings(self, chunks):
         """_fix_sentence_endings(chunks : [string])
 
@@ -97,19 +93,27 @@ class TextWrapper(textwrap.TextWrapper):
             else:
                 i += 1
 
-    def replace(self, text):
-        """ Replace control entities with the proper character 
-            after breaking has occured.
-        """
-        for key, val in self.post_break_replacements.items():
-            text = re.sub(re.escape(key), val, text)
-        return text
-
     def wrap(self, text, initial='', subsequent_indent=None, width=None,
         fix_doublespace=True, fix_sentence_endings=True, drop_whitespace=True):
         """ Mirrored implementation of wrap which replaces characters properly
             and also lets you easily specify indentation on the fly
         """
+        # Handle U+2028 LINE SEPARATOR by recursing
+        if '\u2028' in text:
+            parts = text.split('\u2028')
+            wrapped = []
+            for part in parts:
+                part = part.lstrip()
+                if part:
+                    wrapped += self.wrap(part, initial=initial, subsequent_indent=subsequent_indent,
+                                    width=width, fix_doublespace=fix_doublespace,
+                                    fix_sentence_endings=fix_sentence_endings,
+                                    drop_whitespace=drop_whitespace) 
+                else:
+                    wrapped.append(part)
+                initial = ''
+            return wrapped
+
         # Set indentation
         _width = self.width
         if width != None:
@@ -122,8 +126,9 @@ class TextWrapper(textwrap.TextWrapper):
         self.fix_sentence_endings = fix_sentence_endings
         self.drop_whitespace = drop_whitespace
 
-        # Original implementation
+        # --- Original implementation ----------------------------------
         text = self._munge_whitespace(text)
+        # --- End original implementation ------------------------------
 
         # Maybe remove double (and more) spaces, except when they might be between sentences
         if fix_doublespace:
@@ -133,17 +138,25 @@ class TextWrapper(textwrap.TextWrapper):
         # prevent breaking "Section N.N" and "Appendix X.X"
         text = re.sub("(Section|Appendix|Figure|Table) ", u"\\1\u00A0", text)
 
-        # Replace some characters after splitting has occured
-        parts = self._split(text)
+        # Replace some characters after splitting has occurred
+        # --- Original implementation ----------------------------------
+        chunks = self._split(text)
+        # --- End original implementation ------------------------------
+
+        parts = chunks
         chunks = []
         max_word_len = self.width - len(self.subsequent_indent)
         for chunk in parts:
-            chunk2 = self.replace(chunk)
+            # remove zero-width characters before looking at length
+            chunk2 = re.sub('[\u200B\u200C\u2060\uE060]', '', chunk)
             if len(chunk2) > max_word_len:
-                chunk2 = chunk.replace(u'\u200B', '')
+                # No use trying not to break when we have a chunk longer
+                # than available horizontal space.  Remove wj and nbsp and
+                # try again.
+                chunk2 = re.sub('[\u2060\uE060]', '', chunk).replace('\u00A0', ' ')
                 bits = self._split(chunk2)
                 for bit in bits:
-                    chunk3 = self.replace(bit)
+                    chunk3 = re.sub('[\u2060\uE060]', '', bit).replace('\u00A0', ' ')
                     if len(chunk3) > max_word_len:
                         chunks += self._split(chunk3)
                     else:
@@ -151,15 +164,50 @@ class TextWrapper(textwrap.TextWrapper):
             else:
                 chunks += [ chunk2 ]
         
-        # Original implementation
+        # --- Original implementation ----------------------------------
         if self.fix_sentence_endings:
             self._fix_sentence_endings(chunks)
         wrapped = self._wrap_chunks(chunks)
+        # --- End original implementation ------------------------------
+
         self.width = _width
+        for i, chunk in enumerate(wrapped):
+            wrapped[i] = chunk.replace(u'\uE060', '')
         return wrapped
 
     def fill(self, *args, **kwargs):
-        return "\n".join(self.wrap(*args, **kwargs)).replace(u'\u00A0', ' ')
+        kwargs.pop('fill', None)
+        return "\u2028".join(self.wrap(*args, **kwargs))
+
+
+class TextSplitter(textwrap.TextWrapper):
+    """ Subclass that overrides a few things in the standard implementation """
+    def __init__(self, **kwargs):
+        hyphen_split = kwargs.pop('hyphen_split', False)
+        textwrap.TextWrapper.__init__(self, **kwargs)
+
+        # Override wrapping regex, preserve '/' before linebreak
+        if hyphen_split:
+            self.wordsep_re = re.compile(
+                u'('
+                u'[ \t\n\r\f\v]+|'                                  # any ASCII whitespace
+                u'[^\\s-]*\\w+/(?=[A-Za-z]\\w*)|'                   # forward-slash separated words
+                u'\u200b|'                                          # &zwsp; zero-width space is breakable space
+                u'\u2028|'                                          # &br; unicode 'line separator'
+                u'[^\\s-]*\\w+[^0-9\\s]-(?=\\w+)|'                  # hyphenated words
+                u'''(?<=[\\w\\!"'\\&\\.\\,\\?])-{2,}(?=\\w))'''     # em-dash
+                u'(?![\u00A0|\u2060|\uE060])')                      # UNLESS &nbsp; or &wj; 
+        else:
+            self.wordsep_re = re.compile(
+                u'('
+                u'[ \t\n\r\f\v]+|'                                  # any ASCII whitespace
+                u'[^\\s-]*\\w+/(?=[A-Za-z]\\w*)|'                   # forward-slash separated words
+                u'\u200b|'                                          # &zwsp; zero-width space is breakable space
+                u'\u2028|'                                          # &br; unicode 'line separator'
+                u'''(?<=[\\w\\!"'\\&\\.\\,\\?])-{2,}(?=\\w))'''     # em-dash
+                u'(?![\u00A0|\u2060|\uE060])')                      # UNLESS &nbsp; or &wj; 
+
+        self.wordsep_re_uni = re.compile(self.wordsep_re.pattern, re.U)
 
 
 def justify_inline(left_str, center_str, right_str, width=72):
@@ -191,8 +239,45 @@ def justify_inline(left_str, center_str, right_str, width=72):
             output[i] = right[i]
     return ''.join(output)
 
+# ----------------------------------------------------------------------
+# Other text operations
+
+
+def ascii_split(text):
+    """ We have unicode strings, but we want to split only on the ASCII
+        whitespace characters so that nbsp does not get split.
+    """
+
+    if isinstance(text, type('')):
+        return text.split()
+    return re.split("[ \t\n\r\f\v]+", text)
+
+
+def urlkeep(text, max=72):
+    """ Insert word join XML entities on forward slashes and hyphens
+        in a URL so that it stays on one line
+    """
+    wj_char = u'\uE060'
+    def replacer(match):
+        url = match.group(0)
+        if len(url) > max:
+            return url
+        else:
+            return ( url.replace('/', '/' + wj_char) 
+                        .replace('-', '-' + wj_char) 
+                        .replace(':', ':' + wj_char)
+                    )
+    if 'http://' in text:
+        text = re.sub('http:\S*', replacer, text)
+    if 'https://' in text:
+        text = re.sub('https:\S*', replacer, text)
+    return text
+
+# ----------------------------------------------------------------------
+# Tree operations
+
 def formatXmlWhitespace(tree):
-    """ Traverses an lxml.etree ElementTreeand properly formats whitespace
+    """ Traverses an lxml.etree ElementTree and properly formats whitespace
     
         We replace newlines with single spaces, unless it ends with a
         period then we replace the newline with two spaces.
@@ -210,129 +295,6 @@ def formatXmlWhitespace(tree):
                                re.sub('\.\s*\n\s*', '.  ', \
                                element.tail))
 
-def ascii_split(text):
-    """ We have unicode strings, but we want to split only on the ASCII
-        whitespace characters so that nbsp does not get split.
-    """
-
-    if isinstance(text, type('')):
-        return text.split()
-    return re.split("[ \t\n\r\f\v]+", text)
-
-
-# ----------------------------------------------------------------------
-# Base conversions.
-# From http://tech.vaultize.com/2011/08/python-patterns-number-to-base-x-and-the-other-way/
-
-DEFAULT_DIGITS = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
- 
-def num_to_baseX(num, digits=DEFAULT_DIGITS):
-   if num < 0: return '-' + num_to_baseX(-num)
-   if num == 0: return digits[0]
-   X = len(digits)
-   s = ''
-   while num > 0:
-        s = digits[num % X] + s
-        num //= X
- 
-   return s
- 
-def baseX_to_num(s, digits=DEFAULT_DIGITS):
-   if s[0] == '-': return -1 * baseX_to_num(s[1:])
-   ctopos = dict([(c, pos) for pos, c in enumerate(digits)])
-   X = len(digits)
-   num = 0
-   for c in s: num = num * X + ctopos[c]
-   return num
-
-# ----------------------------------------------------------------------
-# Use the generic base conversion to create list letters
-
-def int2letter(num):
-    return num_to_baseX(num-1, "abcdefghijklmnopqrstuvwxyz")
-
-def int2roman(number):
-    numerals = { 
-        1 : "i", 
-        4 : "iv", 
-        5 : "v", 
-        9 : "ix", 
-        10 : "x", 
-        40 : "xl", 
-        50 : "l", 
-        90 : "xc", 
-        100 : "c", 
-        400 : "cd", 
-        500 : "d", 
-        900 : "cm", 
-        1000 : "m" 
-    }
-    if number > 3999:
-        raise NotImplementedError("Can't handle roman numerals larger than 3999")
-    result = ""
-    for value, numeral in sorted(numerals.items(), reverse=True):
-        while number >= value:
-            result += numeral
-            number -= value
-    return result
-
-
-roman_max_widths = { 1:1,  2:2,  3:3,  4:3,  5:3,  6:3,  7:3,  8:4,  9:4,
-                10:4, 11:4, 12:4, 13:4, 14:4, 15:4, 16:4, 17:4, 18:5, 19:5,
-                20:5, 21:5, 22:5, 23:5, 24:5, 25:5, 26:5, 27:5, 28:6, 29:6, }
-
-def update_roman_max_widths(n):
-    global roman_max_widths
-    if n > 3999:
-        raise NotImplementedError("Can't handle roman numerals larger than 3999")
-    m = len(roman_max_widths)
-    wmax = 0
-    for i in range(n+32):
-        w = len(int2roman(i))
-        if w > wmax:
-            wmax = w
-        if n > m:
-            roman_max_widths[n] = wmax
-
-def num_width(type, num):
-    """
-    Return the largest width taken by the numbering of a list
-    with num items (without punctuation)
-    """
-    global roman_max_widths
-    if   type in ['a','A','c','C',]:
-        return int(math.log(num, 26))+1
-    elif type in ['1','d',]:
-        return int(math.log(num, 10))+1
-    elif type in ['o','O',]:
-        return int(math.log(num, 8))+1
-    elif type in ['x','X',]:
-        return int(math.log(num, 16))+1
-    elif type in ['i','I',]:
-        m = len(roman_max_widths)
-        if num > m:
-            update_roman_max_widths(num)
-        return roman_max_widths[num]
-    else:
-        raise ValueError("Unexpected type argument to num_width(): '%s'" % (type, ))
-
-def urlkeep(text):
-    """ Insert word join XML entities on forward slashes and hyphens
-        in a URL so that it stays on one line
-    """
-    wj_char = u'\u2060'
-    zwsp_char = u'\u200B'
-    def replacer(match):
-        return match.group(0).replace('/', '/' + zwsp_char) \
-                             .replace('/' + zwsp_char + '/' + zwsp_char, '/' + wj_char +'/' + wj_char) \
-                             .replace('-', '-' + wj_char) \
-                             .replace(':', ':' + wj_char)
-                             
-    if 'http://' in text:
-        return re.sub('(?<=http:)\S*', replacer, text)
-    if 'https://' in text:
-        return re.sub('(?<=https:)\S*', replacer, text)
-    return text
 
 def safeReplaceUnicode(tree):
     """ Traverses an lxml.etree ElementTree and replaces unicode characters 
@@ -360,10 +322,59 @@ def safeReplaceUnicode(tree):
                 _replace_unicode_characters(element.attrib[key])
 
 
+def safeTagSlashedWords(tree):
+    """ Traverses an lxml.etree ElementTree and replace words separated
+        by slashes if they are on the list
+    """
+    slashList = {}
+    for element in _slash_replacements:
+        slashList[element] = re.sub(u'/', u'\uE060/\uE060', element)
+        
+    for element in tree.iter():
+        if element.tag == 'artwork':
+            continue
+        if element.text:
+            element.text = _replace_slashed_words(element.text, slashList)
+        if element.tail:
+            element.tail = _replace_slashed_words(element.tail, slashList)
+        # I do not know that this makes any sense
+        # for key in element.attrib.keys():
+        #    element.attrib[key] = \
+        #        _replace_slashed_words(element.attrib[key], slashList)
+
+
+def find_duplicate_ids(schema, tree):
+    # get attributes specified with data type "ID"
+    id_data = schema.xpath("/x:grammar/x:define/x:element//x:attribute/x:data[@type='ID']", namespaces=namespaces)
+    attr = set([ i.getparent().get('name') for i in id_data ])
+    # Check them one by one
+    return find_duplicate_attr_values(attr, tree)
+
+def find_duplicate_html_ids(tree):
+    return find_duplicate_attr_values(['id',], tree)
+
+def find_duplicate_attr_values(attr, tree):
+    dups = []
+    # Check them one by one
+    for a in attr:
+        seen = set()
+        for e in tree.xpath('.//*[@%s]' % a):
+            id = e.get(a)
+            if id != None and id in seen:
+                dups.append((a, id, e))
+            else:
+                seen.add(id)
+    return dups
+
+# ----------------------------------------------------------------------
+# Unicode operations
+
+
 def _replace_unicode_characters(str):
     """ replace those Unicode characters that we do not use internally
         &wj; &zwsp; &nbsp; &nbhy;
     """
+    str = str.replace('\uE060', '\u2060')
     while True:
         match = re.search(u'([^ -\x7e\u2060\u200B\u00A0\u2011\r\n])', str)
         if not match:
@@ -564,24 +575,6 @@ def parse_pi(pi, pis):
         return tmp_dict
     return {}
 
-def safeTagSlashedWords(tree):
-    """ Traverses an lxml.etree ElementTree and replace words seperated
-        by slashes if they are on the list
-    """
-    slashList = {}
-    for element in _slash_replacements:
-        slashList[element] = re.sub(u'/', u'\u200B/\u200B', element)
-        
-    for element in tree.iter():
-        if element.text:
-            element.text = _replace_slashed_words(element.text, slashList)
-        if element.tail:
-            element.tail = _replace_slashed_words(element.tail, slashList)
-        # I do not know that this makes any sense
-        # for key in element.attrib.keys():
-        #    element.attrib[key] = \
-        #        _replace_slashed_words(element.attrib[key], slashList)
-
 
 def _replace_slashed_words(str, slashList):
     """ replace slashed separated words the list with
@@ -606,98 +599,78 @@ _slash_replacements = [
             
 def build_dataurl(mime, data, base64enc=False):
     if base64enc:
-        data = quote(base64.b64encode(data))
+        data = quote(base64.b64encode(data), safe='/~')
         url = "data:%s;base64,%s" % (mime, data)
     else:
-        data = quote(data)
+        data = quote(data, safe='/~')
         url = "data:%s,%s" % (mime, data)
     return url
 
-def normalize_month(month):
-    if len(month) < 3:
-        xml2rfc.log.error("Expected a month name with at least 3 letters, found '%s'" % (month, ))
-    for i, m in enumerate(calendar.month_name):
-        if m and m.lower().startswith(month.lower()):
-            month = '%02d' % (i)
-    if not month.isdigit():
-        xml2rfc.log.error("Expected a month name, found '%s'" % (month, ))
-    return month
-
 namespaces={
     'x':    'http://relaxng.org/ns/structure/1.0',
     'a':    'http://relaxng.org/ns/compatibility/annotations/1.0',
+    'xi':   'http://www.w3.org/2001/XInclude',
+    'svg':  'http://www.w3.org/2000/svg',
     'xml':  'http://www.w3.org/XML/1998/namespace',
+    'xlink':'http://www.w3.org/1999/xlink',
 }
 
-def find_duplicate_ids(schema, tree):
-    dups = []
-    # get attributes specified with data type "ID"
-    id_data = schema.xpath("/x:grammar/x:define/x:element//x:attribute/x:data[@type='ID']", namespaces=namespaces)
-    attr = set([ i.getparent().get('name') for i in id_data ])
-    # Check them one by one
-    for a in attr:
-        seen = set()
-        for e in tree.xpath('.//*[@%s]' % a):
-            id = e.get(a)
-            if id != None and id in seen:
-                dups.append((a, id, e))
-            else:
-                seen.add(id)
-    return dups
+
+def sdict(d):
+    "Create an ordered dict from the given dict, with sorted initial insertion"
+    # For python 3.6 and later, lxml obeys dictionary insertion order for
+    # attributes, for earlier it sorts initial attributes, so we need to
+    # use a regular dict for 3.6 and later
+    if sys.version_info.major == 3 and sys.version_info.minor >= 6:
+        return dict( (k, d[k]) for k in sorted(list(d.keys())) )
+    else:
+        if isinstance(d, OrderedDict):
+            return d
+        return OrderedDict( (k, d[k]) for k in sorted(list(d.keys())) )
+
+# ----------------------------------------------------------------------
+# Element operations
+
 
 def isempty(e):
     "Return True if e has no children and no text content"
-    return not ((len(e) and any([ not isinstance(c, _Comment) for c in e.iterchildren() ]))
-                or (e.text and e.text.strip()) or (e.tail and e.tail.strip()))
+    return not (len(e) or (e.text and e.text.strip()) or (e.tail and e.tail.strip()))
 
 def isblock(e):
     "Return True if e is a block level element"
-    return e.tag in [ 'artwork', 'dl', 'figure', 'ol', 'sourcecode', 't', 'ul', 'blockquote', ]
+    from xml2rfc.writers.base import block_tags
+    return e.tag in block_tags
 
 def iscomment(e):
-    "Return True if e is a comment"
-    return isinstance(e, _Comment)
+    "Return True if e is a comment or PI"
+    return isinstance(e, (_Comment, _ProcessingInstruction))
 
-def hastext(e):
+def hastext(e, ignore=[]):
     "Return a list of text-level immediate children"
     head = [ e.text ] if e.text and e.text.strip() else []
-    items = head + [ c for c in e.iterchildren() if not (isblock(c) or iscomment(c))] + [ c.tail for c in e.iterchildren() if c.tail and c.tail.strip() ]
+    ignore = set(ignore + ['t'])
+    items = ( head + [ c for c in e.iterchildren() if not (isblock(c) or iscomment(c) or (c.tag in ignore))]
+                   + [ c.tail for c in e.iterchildren() if c.tail and c.tail.strip() ] )
     return items
 
-def extract_date(e, today):
-    day = e.get('day')
-    month = e.get('month')
-    year = e.get('year', str(today.year))
-    #
-    if not year.isdigit():
-        xml2rfc.log.warn(e, "Expected a numeric year, but found '%s'" % year)
-        year = today.year
-    year = int(year)
-    if not month:
-        if year == today.year:
-            month = today.month
-    else:
-        if not month.isdigit():
-            month = normalize_month(month)
-        month = int(month)
-    if day:
-        if not day.isdigit():
-            xml2rfc.log.warn(e, "Expected a numeric day, but found '%s'" % day)
-            day = today.day
-        day = int(day)
-    return year, month, day
+def clean_text(s):
+    """Replace internal use code points and various other special code points with plain equivalents"""
+    # spaces
+    s = re.sub(r'[\u00a0\u2028]', ' ', s)
+    # hyphens
+    s = s.replace(r'\u2011', '-')
+    # zero-width
+    s = re.sub(r'[\u200B\u2060\ue060]', '', s)
+    return s.strip()
 
-def format_date(year, month, day, legacy):
-    if month:
-        month = calendar.month_name[month]
-        if day:
-            if legacy:
-                date = '%s %s, %s' % (month, day, year)
-            else:
-                date = '%s %s %s' % (day, month, year)
-        else:
-            date = '%s %s' % (month, year)
-    else:
-        date = '%s' % year
-    return date
+def is_htmlblock(h):
+    return h.tag in set([ 'address', 'article', 'aside', 'blockquote', 'div', 'dl', 'figure',
+        'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'nav', 'ol', 'p', 'pre', 'script', 'section',
+        'table', 'ul', ])
 
+def slugify(s):
+    s = s.strip().lower()
+    s = re.sub(r'[^\w\s/|@=-]', '', s)
+    s = re.sub(r'[-_\s/|@=]+', '_', s)
+    s = s.strip('_')
+    return s
