@@ -13,7 +13,6 @@ from lxml.etree import Element, Comment, CDATA
 
 import xml2rfc
 from xml2rfc import log
-from xml2rfc.util.unicode import unicode_content_tags, unicode_replacements, isascii
 from xml2rfc.utils import hastext, isempty, sdict, slugify, iscomment
 from xml2rfc.writers.base import default_options, BaseV3Writer
 
@@ -316,7 +315,6 @@ class V2v3XmlWriter(BaseV3Writer):
             './/*[self::artwork or self::dl or self::figure or self::ol or self::sourcecode or self::t or self::ul]',
             '//*[@*="yes" or @*="no"]',      # convert old attribute false/true
             '.;pretty_print_prep()',
-            '.;wrap_non_ascii()',
         ]
 
         # Remove any DOCTYPE declaration
@@ -1148,54 +1146,3 @@ class V2v3XmlWriter(BaseV3Writer):
 #             if c.tail != None:
 #                 if c.tail.strip() == '':
 #                     c.tail = None
-
-    def wrap_non_ascii(self, e, p):
-        self.downcode(replacements=unicode_replacements)
-        self.downcode_punctuation()
-        for e in self.tree.iter():
-            def uwrap(text, line): 
-                words = []
-                elements = []
-                ascii = None
-                for word in re.split('(\s+)', text, flags=re.U):
-                    if isascii(word):
-                        words.append(word)
-                    else:
-                        u = self.element('u', line=line)
-                        u.text = word
-                        if ascii is None:
-                            ascii = ''.join(words)
-                            words = []
-                            elements.append(u)
-                        else:
-                            elements[-1].tail = ''.join(words)
-                            words = []
-                            elements.append(u)
-                if words:
-                    if ascii is None:
-                        ascii = ''.join(words)
-                    else:
-                        elements[-1].tail = ''.join(words)
-                return ascii, elements
-            #
-            if e.text and e.tag not in unicode_content_tags:
-                    try:
-                        e.text.encode('ascii')
-                    except UnicodeEncodeError:
-                        e.text, elements = uwrap(e.text, e.sourceline)
-                        if len(e):
-                            c = e[0]
-                            for u in elements:
-                                c.addprevious(u)
-                        else:
-                            for u in elements:
-                                e.append(u)
-            if e.tail and e.getparent().tag not in unicode_content_tags:
-                    try:
-                        e.tail.encode('ascii')
-                    except UnicodeEncodeError:
-                        e.tail, elements = uwrap(e.tail, e.sourceline)
-                        for u in elements:
-                            e.addnext(u)
-                            e = u
-                                
